#include "Config/IniConfigParser.h"
#include "Misc/FileHelper.h"
#include "Misc/Paths.h"

IniConfigParser::IniConfigParser()
{
    ParsedData.Empty();
}

bool IniConfigParser::LoadConfig(const FString& FilePath)
{
    ParsedData.Empty();

    // ȡ INI ļ
    FString FileContent;
    if (!FFileHelper::LoadFileToString(FileContent, *FilePath))
    {
        UE_LOG(LogTemp, Error, TEXT("Failed to load INI file: %s"), *FilePath);
        return false;
    }

    // ÿһнΪֵ
    TArray<FString> Lines;
    FileContent.ParseIntoArrayLines(Lines);

    for (const FString& Line : Lines)
    {
        if (Line.StartsWith(TEXT(";")) || Line.TrimStart().IsEmpty())
        {
            // עͻ
            continue;
        }

        FString Key, Value;
        if (Line.Split(TEXT("="), &Key, &Value))
        {
            ParsedData.Add(Key.TrimStartAndEnd(), Value.TrimStartAndEnd());
        }
    }

    return true;
}

FString IniConfigParser::GetString(const FString& Key, const FString& DefaultValue) const
{
    const FString* FoundValue = ParsedData.Find(Key);
    return FoundValue ? *FoundValue : DefaultValue;
}

int32 IniConfigParser::GetInt(const FString& Key, int32 DefaultValue) const
{
    const FString* FoundValue = ParsedData.Find(Key);
    if (FoundValue)
    {
        return FCString::Atoi(**FoundValue);
    }
    return DefaultValue;
}

float IniConfigParser::GetFloat(const FString& Key, float DefaultValue) const
{
    const FString* FoundValue = ParsedData.Find(Key);
    if (FoundValue)
    {
        return FCString::Atof(**FoundValue);
    }
    return DefaultValue;
}

bool IniConfigParser::GetBool(const FString& Key, bool DefaultValue) const
{
    const FString* FoundValue = ParsedData.Find(Key);
    if (FoundValue)
    {
        return (*FoundValue == TEXT("true") || *FoundValue == TEXT("1"));
    }
    return DefaultValue;
}

TArray<FString> IniConfigParser::GetArray(const FString& Key) const
{
    const FString* FoundValue = ParsedData.Find(Key);
    if (FoundValue)
    {
        TArray<FString> Result;
        FoundValue->ParseIntoArray(Result, TEXT(","), true); // Զŷָ
        return Result;
    }
    return {};
}

TSharedPtr<IConfigParser> IniConfigParser::GetObject(const FString& Key) const
{
    // INI ļ֧ͨǶ׶󣬵ģ֧ "Key.SubKey" Ĳ㼶ṹ
    TMap<FString, FString> SubObjectData;

    for (const auto& Pair : ParsedData)
    {
        if (Pair.Key.StartsWith(Key + TEXT(".")))
        {
            FString SubKey = Pair.Key.RightChop(Key.Len() + 1); // ȥ "Key." 
            SubObjectData.Add(SubKey, Pair.Value);
        }
    }

    if (SubObjectData.Num() > 0)
    {
        TSharedPtr<IniConfigParser> SubParser = MakeShared<IniConfigParser>();
        SubParser->ParsedData = SubObjectData;
        return SubParser;
    }

    return nullptr;
}

void IniConfigParser::PrintConfig() const
{
    for (const auto& Pair : ParsedData)
    {
        UE_LOG(LogTemp, Log, TEXT("%s = %s"), *Pair.Key, *Pair.Value);
    }
}