// Fill out your copyright notice in the Description page of Project Settings.


#include "Core/InJoyConfigManager.h"
#include <Config/JsonConfigParser.h>
#include <Config/IniConfigParser.h>
#include <Config/CSVConfigParser.h>

void TestLoadJsonArray()
{
    JsonConfigParser Parser;

    // һ TArray<UStruct> 洢Ľ
    TArray<FPerson> People;

    //  JSON 鵽 TArray
    if (Parser.LoadArrayStruct(TEXT("Config/People.json"), People))
    {
        for (const FPerson& Person : People)
        {
            UE_LOG(LogTemp, Log, TEXT("Name: %s, Age: %d, Score: %f"), *Person.Name, Person.Age, Person.Score);
        }
    }
    else
    {
        UE_LOG(LogTemp, Error, TEXT("Failed to load JSON array."));
    }
}

void TestSaveJsonArray()
{
    JsonConfigParser Parser;

    // һ TArray<UStruct> 
    TArray<FPerson> People;
    People.Add({ TEXT("John"), 30, 95.5f });
    People.Add({ TEXT("Alice"), 25, 88.0f });
    People.Add({ TEXT("Bob"), 28, 78.0f });

    //  TArray  JSON ļ
    if (Parser.SaveArrayStruct(People, TEXT("Config/GeneratedPeople.json")))
    {
        UE_LOG(LogTemp, Log, TEXT("Saved JSON array successfully."));
    }
    else
    {
        UE_LOG(LogTemp, Error, TEXT("Failed to save JSON array."));
    }
}

void TestIniConfigParser()
{
    IniConfigParser Parser;
    if (Parser.LoadConfig(TEXT("Config/MyConfig.ini")))
    {
        FString Name = Parser.GetString(TEXT("Name"), TEXT("DefaultName"));
        int32 Age = Parser.GetInt(TEXT("Age"), 0);
        bool IsEnabled = Parser.GetBool(TEXT("IsEnabled"), false);

        UE_LOG(LogTemp, Log, TEXT("Name: %s"), *Name);
        UE_LOG(LogTemp, Log, TEXT("Age: %d"), Age);
        UE_LOG(LogTemp, Log, TEXT("IsEnabled: %s"), IsEnabled ? TEXT("True") : TEXT("False"));

        Parser.PrintConfig();
    }
}

void TestCSVConfigParser()
{
    CSVConfigParser Parser;

    if (Parser.LoadConfig(TEXT("Config/MyConfig.csv")))
    {
        // ȡַ
        FString Name = Parser.GetString(TEXT("0.0"), TEXT("DefaultName"));
        UE_LOG(LogTemp, Log, TEXT("Name: %s"), *Name);

        // ȡ
        int32 Age = Parser.GetInt(TEXT("1.1"), 0);
        UE_LOG(LogTemp, Log, TEXT("Age: %d"), Age);

        // ȡ
        float Score = Parser.GetFloat(TEXT("2.2"), 0.0f);
        UE_LOG(LogTemp, Log, TEXT("Score: %f"), Score);

        // ȡֵ
        bool IsActive = Parser.GetBool(TEXT("0.3"), false);
        UE_LOG(LogTemp, Log, TEXT("IsActive: %s"), IsActive ? TEXT("True") : TEXT("False"));

        // ӡ CSV 
        Parser.PrintConfig();
    }
    else
    {
        UE_LOG(LogTemp, Error, TEXT("Failed to load CSV file."));
    }
}